#include <WaveHC.h>
#include <WaveUtil.h>
#include <Brain.h>

// Set up the brain reader, pass it the hardware serial object you want to listen on.
Brain brain(Serial);

SdReader card;    // This object holds the information for the card
FatVolume vol;    // This holds the information for the partition on the card
FatReader root;   // This holds the information for the volumes root directory
FatReader file;   // This object represent the WAV file 
WaveHC wave;      // This is the only wave (audio) object, since we will only play one at a time
long interval = 0;
int house = 0;

/*
 * Define macro to put error messages in flash memory
 */
#define error(msg) error_P(PSTR(msg))

//////////////////////////////////// SETUP
void setup() {
  Serial.begin(9600);

  if (!card.init()) error("card.init");

  // enable optimized read - some cards may timeout
  card.partialBlockRead(true);

  if (!vol.init(card)) error("vol.init");

  if (!root.openRoot(vol)) error("openRoot");
  
  //playGriffindor();
  //playHufflePuff();
  //playRavenClaw();
  //playSlitherin();
  //playThinking1();
  //playThinking2();
  //playTryAgain();
}

//////////////////////////////////// LOOP
void loop() {    
    brain.update();
    // Make sure we have a signal.
    int signalQual = brain.readSignalQuality();
    Serial.println(signalQual);
    if(signalQual < 50) {      
      askAboutHouses();
    }
  }

  void askAboutHouses() {
    playThinking2();
    playAskGriffindor();
    playAskHufflePuff();
    playAskRavenClaw();
    playAskSlitherin();
    playThinking1();    
    makeDecision();
    delay(10000);
  }

  void readMind(int houseIndex) {    
    if (brain.update()) {        
        if (houseIndex != 0) {
          int currInterval = brain.readAttention();
          if (currInterval > interval) {
            interval = currInterval;
            house = houseIndex;
          }        
        }
    }
  }

  void makeDecision() {
    if (house == 1) {
      playGriffindor();
    }
    else if (house == 2) {
      playHufflePuff();
    }
    else if (house == 3) {
      playRavenClaw();
    }
    else if (house == 4) {
      playSlitherin();
    }
    house = 0;
    interval = 0;
  }

/////////////////////////////////// HELPERS
/*
 * print error message and halt
 */
void error_P(const char *str) {
  PgmPrint("Error: ");
  SerialPrint_P(str);
  sdErrorCheck();
  while(1);
}
/*
 * print error message and halt if SD I/O error, great for debugging!
 */
void sdErrorCheck(void) {
  if (!card.errorCode()) return;
  PgmPrint("\r\nSD I/O error: ");
  Serial.print(card.errorCode(), HEX);
  PgmPrint(", ");
  Serial.println(card.errorData(), HEX);
  while(1);
}

/*
 * Play the Griffindor Audio Clip
 */
void playGriffindor(void) {
  char name[10];  
  // copy flash string to RAM
  strcpy_P(name, PSTR("DTMF0.WAV"));    
  playSound(name, 4100, 0);
}

/*
 * Play the Ask Griffindor Audio Clip
 */
void playAskGriffindor(void) {
  char name[10];  
  // copy flash string to RAM
  strcpy_P(name, PSTR("DTMF7.WAV"));    
  playSound(name, 4100, 1);
}

/*
 * Play the HufflePuff Audio Clip
 */
void playHufflePuff(void) {
  char name[10];  
  // copy flash string to RAM
  strcpy_P(name, PSTR("DTMF1.WAV"));    
  playSound(name, 4100, 0);
}

/*
 * Play the Ask HufflePuff Audio Clip
 */
void playAskHufflePuff(void) {
  char name[10];  
  // copy flash string to RAM
  strcpy_P(name, PSTR("DTMF8.WAV"));    
  playSound(name, 4100, 2);
}

/*
 * Play the RavenClaw Audio Clip
 */
void playRavenClaw(void) {
  char name[10];  
  // copy flash string to RAM
  strcpy_P(name, PSTR("DTMF2.WAV"));    
  playSound(name, 1500, 0);
}

/*
 * Play the Ask RavenClaw Audio Clip
 */
void playAskRavenClaw(void) {
  char name[10];  
  // copy flash string to RAM
  strcpy_P(name, PSTR("DTMF9.WAV"));    
  playSound(name, 5100, 3);
}

/*
 * Play the Slitherin Audio Clip
 */
void playSlitherin(void) {
  char name[10];  
  // copy flash string to RAM
  strcpy_P(name, PSTR("DTMF3.WAV"));    
  playSound(name, 2100, 0);
}

/*
 * Play the Ask Slitherin Audio Clip
 */
void playAskSlitherin(void) {
  char name[10];  
  // copy flash string to RAM
  strcpy_P(name, PSTR("DTMF10.WAV"));    
  playSound(name, 4500, 4);
}

/*
 * Play the Thinking1 Audio Clip
 */
void playThinking1(void) {
  char name[10];  
  // copy flash string to RAM
  strcpy_P(name, PSTR("DTMF4.WAV"));    
  playSound(name, 7100, 0);
}

/*
 * Play the Thinking2 Audio Clip
 */
void playThinking2(void) {
  char name[10];  
  // copy flash string to RAM
  strcpy_P(name, PSTR("DTMF5.WAV"));    
  playSound(name, 19100, 0);
}

/*
 * Play the TryAgain Audio Clip
 */
void playTryAgain(void) {
  char name[10];  
  // copy flash string to RAM
  strcpy_P(name, PSTR("DTMF6.WAV"));    
  playSound(name, 10100, 0);
}

/*
 * Play whatever sound file name is passed through, for the duration specified in milliseconds
 */
void playSound(char* name, int millisecs, int houseIndex) {
  // start time
    uint32_t t = millis();
  // open file by name
  if (!file.open(root, name)) error("open by name");     
  // create wave and start play
  if (!wave.create(file))error("wave.create");
  wave.play();    
  // stop after PLAY_TIME ms
    while((millis() - t) < millisecs) {
      if (houseIndex != 0) {
        readMind(houseIndex);
      }
    }
    wave.stop();
  // check for play errors
  sdErrorCheck();    
  PgmPrintln("Done");
}

